// FtCtrl.cpp: CFtCtrl NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FtCtrl.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#pragma comment(lib, "FTD2XX.lib")

#include "FtException.h"

/* ========================================================================
	RXgN^EfXgN^
 ======================================================================== */

CFtCtrl::CFtCtrl()
{
	m_hFt				= INVALID_HANDLE_VALUE;
}

CFtCtrl::~CFtCtrl()
{
//	ASSERT(m_hFt == INVALID_HANDLE_VALUE);
	if(m_hFt != INVALID_HANDLE_VALUE)
	{
		Close();
	}
}

/* ========================================================================
	PrivateEProtected
 ======================================================================== */

FT_STATUS CFtCtrl::ValidStatus(FT_STATUS nStatus, const char* szFunc, BOOL fThrowException)
{
#ifdef FTCTRL_THROW_EXCEPTION
	if(fThrowException != 0)
	{
		if((nStatus != FT_OK) && (nStatus != FT_DEVICE_NOT_FOUND))
		{
			throw(new CFtException(nStatus, szFunc));
		}
	}
#endif // FTCTRL_THROW_EXCEPTION

	return	nStatus;
}



/* ========================================================================
	Public
 ======================================================================== */

void CFtCtrl::RemoveAllDeviceList(CFtDevInfoList* pListInfo)
{
	while(pListInfo->IsEmpty() == 0)
	{
		delete	pListInfo->RemoveHead();
	}
}

FT_STATUS CFtCtrl::EnumDevices(CFtDevInfoList* pListInfo)
{
	ASSERT(pListInfo->IsEmpty() != 0);

	FT_STATUS	nSta;
	int	i;
	const char*	pszDesc[MAX_ENUM_DEVICES + 1];
	char		szDesc[MAX_ENUM_DEVICES][MAX_DESC_LEN];

	// [NGA
	for(i = 0; i < MAX_ENUM_DEVICES; i++)
	{
		pszDesc[i]	= &szDesc[i][0];
		memset(&szDesc[i], '\0', sizeof(szDesc[0]));
	}
	pszDesc[MAX_ENUM_DEVICES]	= NULL;

	// foCX擾
	if(FT_SUCCESS(nSta = ListDevices(pszDesc, szDesc, FT_LIST_ALL | FT_OPEN_BY_DESCRIPTION)))
	{
		// VAԍ擾
		char	szSer[MAX_SER_LEN];
		for(i = 0; i < MAX_ENUM_DEVICES; i++)
		{
			if(szDesc[i][0] == '\0')
				break;

			if(FT_SUCCESS(nSta = FT_ListDevices((LPVOID)i, szSer, FT_LIST_BY_INDEX | FT_OPEN_BY_SERIAL_NUMBER)))
			{
				pListInfo->AddTail(new CFtDevInfo(i, szDesc[i], szSer));
			}
		}
	}

	return	nSta;
}


/* ========================================================================
	Public (API wrapper)
 ======================================================================== */

FT_STATUS CFtCtrl::Open(
	int deviceNumber
	)
{
	return	ValidStatus(
		FT_Open(deviceNumber, &m_hFt), 
		"FT_Open"
		);
}

FT_STATUS CFtCtrl::OpenEx(
	PVOID pArg1,
	DWORD Flags
	)
{
	return	ValidStatus(
		FT_OpenEx(pArg1, Flags, &m_hFt), 
		"FT_OpenEx"
		);
}

FT_STATUS CFtCtrl::ListDevices(
	PVOID pArg1,
	PVOID pArg2,
	DWORD Flags
	)
{
	return	ValidStatus(
		FT_ListDevices(pArg1, pArg2, Flags), 
		"FT_ListDevices"
		);
}

FT_STATUS CFtCtrl::Close(
	)
{
	FT_STATUS	nStatus	= 
		ValidStatus(
		FT_Close(m_hFt), 
		"FT_Close",
		FALSE				// OsȂ
		);

	if(nStatus == FT_OK)
	{
		m_hFt	= INVALID_HANDLE_VALUE;
	}

	return	nStatus;
}

FT_STATUS CFtCtrl::Read(
	LPVOID lpBuffer,
	DWORD nBufferSize,
	LPDWORD lpBytesReturned
	)
{
	return	ValidStatus(
		FT_Read(m_hFt, lpBuffer, nBufferSize, lpBytesReturned), 
		"FT_Read"
		);
}

FT_STATUS CFtCtrl::Write(
//	LPVOID lpBuffer,
	const VOID* lpBuffer,
	DWORD nBufferSize,
	LPDWORD lpBytesWritten
	)
{
	return	ValidStatus(
		FT_Write(m_hFt, (LPVOID)lpBuffer, nBufferSize, lpBytesWritten), 
		"FT_Write"
		);
}

FT_STATUS CFtCtrl::IoCtl(
	DWORD dwIoControlCode,
	LPVOID lpInBuf,
	DWORD nInBufSize,
	LPVOID lpOutBuf,
	DWORD nOutBufSize,
	LPDWORD lpBytesReturned,
	LPOVERLAPPED lpOverlapped
	)
{
	return	ValidStatus(
		FT_IoCtl(m_hFt, dwIoControlCode, lpInBuf, nInBufSize, lpOutBuf, nOutBufSize, lpBytesReturned, lpOverlapped), 
		"FT_IoCtl"
		);
}

FT_STATUS CFtCtrl::SetBaudRate(
	ULONG BaudRate
	)
{
	return	ValidStatus(
		FT_SetBaudRate(m_hFt, BaudRate), 
		"FT_SetBaudRate"
		);
}

FT_STATUS CFtCtrl::SetDivisor(
	USHORT Divisor
	)
{
	return	ValidStatus(
		FT_SetDivisor(m_hFt, Divisor), 
		"FT_SetDivisor"
		);
}

FT_STATUS CFtCtrl::SetDataCharacteristics(
	UCHAR WordLength,
	UCHAR StopBits,
	UCHAR Parity
	)
{
	return	ValidStatus(
		FT_SetDataCharacteristics(m_hFt, WordLength, StopBits, Parity), 
		"FT_SetDataCharacteristics"
		);
}

FT_STATUS CFtCtrl::SetFlowControl(
	USHORT FlowControl,
	UCHAR XonChar,
	UCHAR XoffChar
	)
{
	return	ValidStatus(
		FT_SetFlowControl(m_hFt, FlowControl, XonChar, XoffChar), 
		"FT_SetFlowControl"
		);
}

FT_STATUS CFtCtrl::ResetDevice()
{
	return	ValidStatus(
		FT_ResetDevice(m_hFt), 
		"FT_ResetDevice"
		);
}

FT_STATUS CFtCtrl::SetDtr()
{
	return	ValidStatus(
		FT_SetDtr(m_hFt), 
		"FT_SetDtr"
		);
}

FT_STATUS CFtCtrl::ClrDtr()
{
	return	ValidStatus(
		FT_ClrDtr(m_hFt), 
		"FT_ClrDtr"
		);
}

FT_STATUS CFtCtrl::SetRts()
{
	return	ValidStatus(
		FT_SetRts(m_hFt), 
		"FT_SetRts"
		);
}

FT_STATUS CFtCtrl::ClrRts()
{
	return	ValidStatus(
		FT_SetRts(m_hFt), 
		"FT_SetRts"
		);
}

FT_STATUS CFtCtrl::GetModemStatus(
	ULONG *pModemStatus
	)
{
	return	ValidStatus(
		FT_GetModemStatus(m_hFt, pModemStatus), 
		"FT_GetModemStatus"
		);
}

FT_STATUS CFtCtrl::SetChars(
	UCHAR EventChar,
	UCHAR EventCharEnabled,
	UCHAR ErrorChar,
	UCHAR ErrorCharEnabled
	)
{
	return	ValidStatus(
		FT_SetChars(m_hFt, EventChar, EventCharEnabled, ErrorChar, ErrorCharEnabled), 
		"FT_SetChars"
		);
}

FT_STATUS CFtCtrl::Purge(
	ULONG Mask
	)
{
	return	ValidStatus(
		FT_Purge(m_hFt, Mask), 
		"FT_Purge"
		);
}

FT_STATUS CFtCtrl::SetTimeouts(
	ULONG ReadTimeout,
	ULONG WriteTimeout
	)
{
	return	ValidStatus(
		FT_SetTimeouts(m_hFt, ReadTimeout, WriteTimeout), 
		"FT_SetTimeouts"
		);
}

FT_STATUS CFtCtrl::GetQueueStatus(
	DWORD *dwRxBytes
	)
{
	return	ValidStatus(
		FT_GetQueueStatus(m_hFt, dwRxBytes), 
		"FT_GetQueueStatus"
		);
}

FT_STATUS CFtCtrl::SetEventNotification(
	DWORD Mask,
	PVOID Param
	)
{
	return	ValidStatus(
		FT_SetEventNotification(m_hFt, Mask, Param), 
		"FT_SetEventNotification"
		);
}

FT_STATUS CFtCtrl::GetStatus(
	DWORD *dwRxBytes,
	DWORD *dwTxBytes,
	DWORD *dwEventDWord
	)
{
	return	ValidStatus(
		FT_GetStatus(m_hFt, dwRxBytes, dwTxBytes, dwEventDWord), 
		"FT_GetStatus"
		);
}

FT_STATUS CFtCtrl::SetBreakOn()
{
	return	ValidStatus(
		FT_SetBreakOn(m_hFt), 
		"FT_SetBreakOn"
		);
}

FT_STATUS CFtCtrl::SetBreakOff()
{
	return	ValidStatus(
		FT_SetBreakOff(m_hFt), 
		"FT_SetBreakOff"
		);
}

FT_STATUS CFtCtrl::SetWaitMask(
	DWORD Mask
	)
{
	return	ValidStatus(
		FT_SetWaitMask(m_hFt, Mask), 
		"FT_SetWaitMask"
		);
}

FT_STATUS CFtCtrl::WaitOnMask(
	DWORD *Mask
	)
{
	return	ValidStatus(
		FT_WaitOnMask(m_hFt, Mask), 
		"FT_WaitOnMask"
		);
}

FT_STATUS CFtCtrl::GetEventStatus(
	DWORD *dwEventDWord
	)
{
	return	ValidStatus(
		FT_GetEventStatus(m_hFt, dwEventDWord), 
		"FT_GetEventStatus"
		);
}

FT_STATUS CFtCtrl::ReadEE(
	DWORD dwWordOffset,
	LPWORD lpwValue
	)
{
	return	ValidStatus(
		FT_ReadEE(m_hFt, dwWordOffset, lpwValue), 
		"FT_ReadEE"
		);
}

FT_STATUS CFtCtrl::WriteEE(
	DWORD dwWordOffset,
	WORD wValue
	)
{
	return	ValidStatus(
		FT_WriteEE(m_hFt, dwWordOffset, wValue), 
		"FT_WriteEE"
		);
}

FT_STATUS CFtCtrl::EraseEE()
{
	return	ValidStatus(
		FT_EraseEE(m_hFt), 
		"FT_EraseEE"
		);
}

// FT8U232/245BM
FT_STATUS CFtCtrl::EE_Program(
	PFT_PROGRAM_DATA pData
	)
{
	return	ValidStatus(
		FT_EE_Program(m_hFt, pData), 
		"FT_EE_Program"
		);
}

FT_STATUS CFtCtrl::EE_Read(
	PFT_PROGRAM_DATA pData
	)
{
	return	ValidStatus(
		FT_EE_Read(m_hFt, pData), 
		"FT_EE_Read"
		);
}

FT_STATUS CFtCtrl::EE_UASize(
	LPDWORD lpdwSize
	)
{
	return	ValidStatus(
		FT_EE_UASize(m_hFt, lpdwSize), 
		"FT_EE_UASize"
		);
}

FT_STATUS CFtCtrl::EE_UAWrite(
	PUCHAR pucData,
	DWORD dwDataLen
	)
{
	return	ValidStatus(
		FT_EE_UAWrite(m_hFt, pucData, dwDataLen), 
		"FT_EE_UAWrite"
		);
}

FT_STATUS CFtCtrl::EE_UARead(
	PUCHAR pucData,
	DWORD dwDataLen,
	LPDWORD lpdwBytesRead
	)
{
	return	ValidStatus(
		FT_EE_UARead(m_hFt, pucData, dwDataLen, lpdwBytesRead), 
		"FT_EE_UARead"
		);
}

FT_STATUS CFtCtrl::SetLatencyTimer(
	UCHAR ucLatency
	)
{
	return	ValidStatus(
		FT_SetLatencyTimer(m_hFt, ucLatency), 
		"FT_SetLatencyTimer"
		);
}

FT_STATUS CFtCtrl::GetLatencyTimer(
	PUCHAR pucLatency
	)
{
	return	ValidStatus(
		FT_GetLatencyTimer(m_hFt, pucLatency), 
		"FT_GetLatencyTimer"
		);
}

FT_STATUS CFtCtrl::SetBitMode(
	UCHAR ucMask,
	UCHAR ucEnable
	)
{
	return	ValidStatus(
		FT_SetBitMode(m_hFt, ucMask, ucEnable), 
		"FT_SetBitMode"
		);
}

FT_STATUS CFtCtrl::GetBitMode(
	PUCHAR pucMode
	)
{
	return	ValidStatus(
		FT_GetBitMode(m_hFt, pucMode), 
		"FT_GetBitMode"
		);
}

FT_STATUS CFtCtrl::SetUSBParameters(
	ULONG ulInTransferSize,
	ULONG ulOutTransferSize
	)
{
	return	ValidStatus(
		FT_SetUSBParameters(m_hFt, ulInTransferSize, ulOutTransferSize), 
		"FT_SetUSBParameters"
		);
}

FT_STATUS CFtCtrl::GetDeviceInfo(
	FT_DEVICE *lpftDevice,
	LPDWORD lpdwID,
	PCHAR SerialNumber,
	PCHAR Description,
	LPVOID Dummy
	)
{
	return	ValidStatus(
		FT_GetDeviceInfo(m_hFt, lpftDevice, lpdwID, SerialNumber, Description, Dummy), 
		"FT_GetDeviceInfo"
		);
}


