// LcdCtrl.cpp: CLcdCtrl NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FTLcd.h"
#include "LcdCtrl.h"
#include "mmsystem.h"

#pragma comment(lib, "winmm.lib")


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


const LCDCMD_INFO	g_CmdInfo[]	= {
	{ FALSE,	0x01,	0x00,	1530	},		// LCDCMD_CLEAR_DISPLAY
	{ FALSE,	0x02,	0x00,	1530	},		// LCDCMD_CURSOR_AT_HOME
	{ FALSE,	0x04,	0x03,	37		},		// LCDCMD_ENTRY_MODE_SET
	{ FALSE,	0x08,	0x07,	37		},		// LCDCMD_DISPLAY_MODE_SET
	{ FALSE,	0x10,	0x0c,	37		},		// LCDCMD_CURSOR_MODE_SET
	{ FALSE,	0x20,	0x1c,	37		},		// LCDCMD_INITIAL_SET
	{ FALSE,	0x40,	0x3f,	37		},		// LCDCMD_SET_CGRAM_ADDR
	{ FALSE,	0x80,	0x7f,	37		},		// LCDCMD_SET_DDRAM_ADDR
	{ TRUE,		0x00,	0xff,	1530	},		// LCDCMD_DATA_WRITE
};


#include "FtException.h"


//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CLcdCtrl::CLcdCtrl()
{
	m_dwLastExecutedTick	= timeGetTime();
	m_nWait		= 0;
	m_fOpen		= FALSE;
}

CLcdCtrl::~CLcdCtrl()
{

}

BOOL CLcdCtrl::Open(int nDevID)
{
	if(m_fOpen == FALSE)
	{
		try
		{
			if(FT_SUCCESS(m_ft.Open(nDevID)) != 0)
			{
				const BYTE	bWrt1[]	= { 0x00 };
				const BYTE	bWrt2[]	= { 0x03, 0x03 | LCD_E1, 0x03 };
				const BYTE	bWrt3[]	= { 0x02, 0x02 | LCD_E1, 0x02 };
				DWORD		dwWritten;

				m_ft.ResetDevice();
				m_ft.Purge(FT_PURGE_RX | FT_PURGE_TX);
				m_ft.SetBaudRate(38400);

				m_ft.Write(bWrt1, sizeof(bWrt1), &dwWritten);
				m_ft.SetBitMode(LCD_BITMODE_WRITE, 1);

				// Initialize LCD
				WaitMSec(15);		// 15ms
				m_ft.Write(&bWrt2, sizeof(bWrt2), &dwWritten);
				WaitMSec(5);		// 4.1ms
				m_ft.Write(&bWrt2, sizeof(bWrt2), &dwWritten);
				WaitMSec(100);		// 100ms
				m_ft.Write(&bWrt2, sizeof(bWrt2), &dwWritten);
				WaitMSec(100);		// 100ms
				m_ft.Write(&bWrt3, sizeof(bWrt3), &dwWritten);

				WriteCommand(LCDCMD_DISPLAY_MODE_SET, 0x04);
				ClearScreen();

				m_fOpen	= TRUE;
				return	TRUE;
			}
		}
		catch(CFtException* e)
		{
			e->ReportError();
			e->Delete();
		}
	}

	return	FALSE;
}

void CLcdCtrl::Close()
{
	if(m_fOpen == TRUE)
	{
		try
		{
			m_ft.SetBitMode(0x00, 0);
			m_ft.Close();
			m_fOpen	= FALSE;
		}
		catch(CFtException* e)
		{
			e->ReportError();
			e->Delete();
		}
	}
}

void CLcdCtrl::WaitMSec(int nMSec)
{
	if(nMSec > 10)
	{
		TRACE1("Sleep %d ms\n", nMSec);
		Sleep(nMSec);		// Ƃ肠
	}
}

BOOL CLcdCtrl::WriteCommand(LCDCMD nCmd, BYTE bData)
{
	if((0 <= nCmd) && (nCmd <= (sizeof(g_CmdInfo) / sizeof(g_CmdInfo[0]))))
	{
		DWORD	dwDiff;
		const LCDCMD_INFO*	pInfo;
		
		pInfo	= &g_CmdInfo[nCmd];
		dwDiff	= timeGetTime() - m_dwLastExecutedTick;

		if(dwDiff < (DWORD)m_nWait)
		{
			WaitMSec((DWORD)m_nWait - dwDiff);
		}

		try
		{
			BYTE	bWrt[6];
			BYTE	bCmd;
			BYTE	bCmdLo;
			BYTE	bCmdHi;
			BYTE	bCtlbit;
			DWORD	dwWritten;

			bCmd	= (bData & pInfo->bMask) | pInfo->bCmd;
			bCmdLo	= bCmd & 0x0f;
			bCmdHi	= (bCmd >> 4) & 0x0f;
			bCtlbit	= (BYTE)((pInfo->fRS == FALSE) ? 0x00 : LCD_RS);

			bWrt[0]	= bCtlbit | bCmdHi;
			bWrt[1]	= bCtlbit | bCmdHi | (BYTE)LCD_E1;
			bWrt[2]	= bCtlbit | bCmdHi;

			bWrt[3]	= bCtlbit | bCmdLo;
			bWrt[4]	= bCtlbit | bCmdLo | (BYTE)LCD_E1;
			bWrt[5]	= bCtlbit | bCmdLo;

			if(FT_SUCCESS(m_ft.Write(bWrt, sizeof(bWrt), &dwWritten)) != 0)
			{
				m_dwLastExecutedTick	= timeGetTime();
				m_nWait	= (pInfo->nWait / 1000);
	
				return	TRUE;
			}
		}
		catch(CFtException* e)
		{
			e->ReportError();
			e->Delete();
		}
	}

	return	FALSE;
}

BOOL CLcdCtrl::WriteChar(char ch)
{
	return	WriteCommand(LCDCMD_DATA_WRITE, ch);
}

BOOL CLcdCtrl::WriteString(const char* szData)
{
	char	ch;

	while((ch = *szData) != '\0')
	{
		if(WriteChar(ch) == FALSE)
			return	FALSE;

		szData++;
	}

	return	TRUE;
}

BOOL CLcdCtrl::LocateCursor(int px, int py)
{
	return	WriteCommand(LCDCMD_SET_DDRAM_ADDR, py * 0x40 + px);
}

BOOL CLcdCtrl::SetFont(int nIndex, const BYTE* pbData, int nLen)
{
	if(WriteCommand(LCDCMD_SET_CGRAM_ADDR, nIndex * 8) == FALSE)
		return	FALSE;
	
	int	i;
	for(i = 0; i < nLen; i++)
	{
		if(WriteCommand(LCDCMD_DATA_WRITE, pbData[i]) == FALSE)
			return	FALSE;
	}

	return	TRUE;
}

BOOL CLcdCtrl::ClearScreen()
{
	return	WriteCommand(LCDCMD_CLEAR_DISPLAY, 0x00);
}

int CLcdCtrl::GetKeyData()
{
	try
	{
		BYTE	bData;
		m_ft.SetBitMode(LCD_BITMODE_GETKEY, 1);
		m_ft.GetBitMode(&bData);
		m_ft.SetBitMode(LCD_BITMODE_WRITE, 1);

		return	(int)(bData & LCD_KEYDATA_MASK);
	}
	catch(CFtException* e)
	{
		e->ReportError();
		e->Delete();
	}

	return	-1;
}

